//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

datablock StaticShapeData(BrookTroutShape)
{
   // The category variable determins where the item
   // shows up in the mission editor's creator tree.
   // Path is relative to where the script is executed from.
   category = "Misc";
   shapeFile = "~/data/shapes/naturepack/goldenshiner.dts";
   isPlaying = 1;
};

function StaticShapeData::create(%block)
{
   // The mission editor invokes this method when it
   // wants to create an object of the given datablock
   // type.  You only need one of these methods for any
   // class/datablock type (in this case StaticShape).
   
   %obj = new StaticShape()
   {
      dataBlock = %block;
   };
   return(%obj);
}

//----------------------------------------------------------------------------

datablock DebrisData( BrookTroutDebris )
{
   explodeOnMaxBounce = false;

   elasticity = 0.15;
   friction = 0.5;

   lifetime = 4.0;
   lifetimeVariance = 0.0;

   minSpinSpeed = 40;
   maxSpinSpeed = 600;

   numBounces = 5;
   bounceVariance = 0;

   staticOnMaxBounce = true;
   gravModifier = 1.0;

   useRadiusMass = true;
   baseRadius = 1;

   velocity = 20.0;
   velocityVariance = 12.0;
};             

datablock PlayerData(BrookTrout)
{
   className = Fish;
   shapeFile = "~/data/shapes/naturepack/birdmount.dts";
   computeCRC = true;
   debrisShapeName = "~/data/shapes/naturepack/goldenshiner.dts";
   debris = BrookTroutDebris;
   aiAvoidThis = true;

   mass = 90;
   drag = 0.3;
   maxdrag = 0.4;
   density = 10;
   maxDamage = 1;
   maxEnergy =  60;
   repairRate = 0;
   energyPerDamagePoint = 0;

   rechargeRate = 0.256;

   runForce = 48 * 90;
   runEnergyDrain = 0;
   minRunEnergy = 0;
   maxForwardSpeed = 7; //14;
   maxBackwardSpeed = 6; //13;
   maxSideSpeed = 6; //13;

   maxUnderwaterForwardSpeed = 3.5; //8.4;
   maxUnderwaterBackwardSpeed = 2.5; //7.8;
   maxUnderwaterSideSpeed = 2.5; //7.8;

   jumpForce = 8.3 * 90;
   jumpEnergyDrain = 0;
   minJumpEnergy = 0;
   jumpDelay = 15;

   recoverDelay = 9;
   recoverRunForceScale = 1.2;

   minImpactSpeed = 45;
   speedDamageScale = 0.4;

   boundingBox = "0.1 0.1 0.1";
   pickupRadius = 0.75;
   
   // Controls over slope of runnable/jumpable surfaces
   runSurfaceAngle  = 90; //70;
   jumpSurfaceAngle = 80;

   minJumpSpeed = 20;
   maxJumpSpeed = 30;

   horizMaxSpeed = 68;
   horizResistSpeed = 33;
   horizResistFactor = 0.35;

   upMaxSpeed = 80;
   upResistSpeed = 25;
   upResistFactor = 0.3;
   
   footstepSplashHeight = 0.35;
   
   groundImpactMinSpeed    = 10.0;
   groundImpactShakeFreq   = "4.0 4.0 4.0";
   groundImpactShakeAmp    = "1.0 1.0 1.0";
   groundImpactShakeDuration = 0.8;
   groundImpactShakeFalloff = 10.0;
};

//----------------------------------------------------------------------------
// Fish Datablock methods
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------

function Fish::onAdd(%this,%obj)
{
   // Vehicle timeout
   %obj.mountVehicle = false;

   // Default dynamic armor stats
   %obj.setRechargeRate(%this.rechargeRate);
   %obj.setRepairRate(0);

   %fish = new StaticShape() {
      dataBlock = BrookTroutShape;
      scale = "0.5 0.5 0.5";
   };

   %obj.fishshape = %fish;
   %fish.fish = %obj;
   
   %obj.MountObject(%fish, 0);

   %fish.playThread(0,"swimfst");
}

function Fish::onRemove(%this, %obj)
{
   if (isObject(%obj.fishshape))
   {
      %obj.unmountObject(%obj.fishshape);
      %obj.fishshape.schedule(300, "delete");
   }
}

//----------------------------------------------------------------------------

function Fish::onCollision(%this,%obj,%col,%vec,%speed)
{
   //echo("Fish::onCollision()");
   if (%obj.getState() $= "Dead")
      return;
}

function Fish::onImpact(%this, %obj, %collidedObject, %vec, %vecLen)
{
   //echo("Fish::onImpact()");
   %obj.damage(0, VectorAdd(%obj.getPosition(),%vec),
      %vecLen * %this.speedDamageScale, "Impact");
}


//----------------------------------------------------------------------------

function Fish::damage(%this, %obj, %sourceObject, %position, %damage, %damageType)
{
   //echo("Fish::damage");
   if (%obj.getState() $= "Dead")
      return;
   %obj.applyDamage(%damage);
}

function Fish::onDamage(%this, %obj, %delta)
{
   //echo("Fish::onDamage");
   // This method is invoked by the ShapeBase code whenever the 
   // object's damage level changes.
   if (%delta > 0 && %obj.getState() !$= "Dead") {

      // Increment the flash based on the amount.
      %flash = %obj.getDamageFlash() + ((%delta / %this.maxDamage) * 2);
      if (%flash > 0.75)
         %flash = 0.75;
      %obj.setDamageFlash(%flash);
   }
}

function Fish::onDisabled(%this,%obj,%state)
{
   //echo("Fish::onDisabled");
   // The FlyingInsect object sets the "disabled" state when damage exceeds
   // it's maxDamage value.  This is method is invoked by ShapeBase
   // state mangement code.
   %obj.fishshape.stopThread(0);
   // Schedule corpse removal.  Just keeping the place clean.
   %obj.schedule(30, "delete");
}

//-----------------------------------------------------------------------------

function Fish::onEnterLiquid(%this, %obj, %coverage, %type)
{
   //echo("Fish::onEnterLiquid");
   switch(%type)
   {
      case 0: //Water
      case 1: //Ocean Water
      case 2: //River Water
      case 3: //Stagnant Water
      case 4: //Lava
         %obj.setDamageDt(%this, $DamageLava, "Lava");
      case 5: //Hot Lava
         %obj.setDamageDt(%this, $DamageHotLava, "Lava");
      case 6: //Crusty Lava
         %obj.setDamageDt(%this, $DamageCrustyLava, "Lava");
      case 7: //Quick Sand
   }
}

function Fish::onLeaveLiquid(%this, %obj, %type)
{
   %obj.clearDamageDt();
}


//-----------------------------------------------------------------------------

function Fish::onTrigger(%this, %obj, %triggerNum, %val)
{
   // This method is invoked when the player receives a trigger
   // move event.  The player automatically triggers slot 0 and
   // slot one off of triggers # 0 & 1.  Trigger # 2 is also used
   // as the jump key.
}

// The AIPlayer C++ class implements the following methods.
//
// AIPlayer::stopMove();
//    Stop the player where he is.
//
// AIPlayer::clearAim();
//    Clear any current aim location or aim object.  The player will
//    maintain his current orientation, or face towards his destination
//    if moving.
//
// AIPlayer::setMoveSpeed( float );
//    Set the speed (0-1) at which the player moves.
//
// AIPlayer::setMoveDestination( "x y z" );
//    Set the destination point to move towards. The Z location is ignored
//    though the player will face upwards (or downards) towards the destination
//    if no other aim target is set.
//
// AIPlayer::getMoveDestination();
//    Returns the current destination point.
//
// AIPlayer::setAimLocation( \"x y z\" );
//    Set a point to look at.  The player maintains his orientation towards
//    this point even while moving towards a destination point.
//
// AIPlayer::setAimObject( obj );
//    Set an object to look at.  The player maintains his orientation towards
//    this object, even while moving towards a destination point. The aim
//    object can also be moving.
//
// AIPlayer::getAimLocation();
//    Returns the current point the player is looking at. If the player has
//    an aimObject set, this would be the current position of the target
//    object.
//
// AIPlayer::getAimObject();
//    Returns the current target object, if there is one.

//-----------------------------------------------------------------------------
// Callback Handlers
//-----------------------------------------------------------------------------

function Fish::onStuck(%this,%obj) 
{
   // Invoked if the player is stuck while moving
   // This method is currently not being invoked correctly.
   echo( "I'm stuck" );
}

function Fish::onReachDestination(%this,%obj)
{
   // Invoked when the player arrives at his destination point.
   //echo( "onReachDestination" );
   
   // figure out what the next marker is in the path
   %obj.curMarker = %obj.curMarker + 1;
   if (%obj.curMarker == %obj.numMarkers)
      %obj.curMarker = 0;

   
   
   // go to the next marker
   %marker = %obj.marker[%obj.curMarker];
   %matrix = %marker.getTransform();
   %pos = getWords(%matrix, 0, 2);

   %obj.setAimLocation(%pos);
   %obj.setMoveDestination(%pos);
}

function Fish::onTargetEnterLOS(%this,%obj)
{
   // If an aim target object is set, this method is invoked when
   // that object becomes visible.
   //echo( "onTargetEnterLOS" );
}

function Fish::onTargetExitLOS(%this,%obj)
{
   // If an aim target object is set, this method is invoked when
   // the object is not longer visible.
   //echo( "onTargetExitLOS" );
}

//-----------------------------------------------------------------------------