//*****************************************************************************
// Torque -- HLSL procedural shader
//*****************************************************************************

// Dependencies:
#include "shaders/common/torque.hlsl"

// Features:
// Vert Position
// Reflect Cube
// Visibility
// Fog
// HDR Output

struct ConnectData
{
   float3 reflectVec      : TEXCOORD0;
   float3 wsPosition      : TEXCOORD1;
};


struct Fragout
{
   float4 col : COLOR0;
};


//-----------------------------------------------------------------------------
// Main
//-----------------------------------------------------------------------------
Fragout main( ConnectData IN,
              uniform samplerCUBE cubeMap         : register(S0),
              uniform float     visibility      : register(C0),
              uniform float4    fogColor        : register(C1),
              uniform float3    eyePosWorld     : register(C2),
              uniform float3    fogData         : register(C3)
)
{
   Fragout OUT;

   // Vert Position
   
   // Reflect Cube
   OUT.col = texCUBE( cubeMap, IN.reflectVec );
   
   // Visibility
   OUT.col.w *= visibility;
   
   // Fog
   float fogAmount = saturate( computeSceneFog( eyePosWorld, IN.wsPosition, fogData.r, fogData.g, fogData.b ) );
   OUT.col = lerp( fogColor, OUT.col, fogAmount );
   
   // HDR Output
   hdrEncode( OUT.col );
   

   return OUT;
}
